####################################################
# original function 'VARselect' in package 'vars'
# modified for direct forecasting by Nikolay Robinzonov
# Date: 2007-10-01
####################################################
#
#
#
#                 I T E R A T I V E
#
#
# restrict()  - modified copy from 'restrict' in package 'vars'   (!)
####################################################
restrict <- function (x, method = c("ser", "manual"), thresh = 2, resmat = NULL) 
{
    if (!(class(x) == "varest")) {
        stop("\nPlease provide an object of class 'varest', generated by 'var()'.\n")
    }
    method <- match.arg(method)
    thresh <- abs(thresh)
    K <- x$K
    p <- x$p
    datasub <- x$datamat[, -c(1:K)]
    namesall <- colnames(datasub)
    yendog <- x$datamat[, c(1:K)]
    sample <- x$obs
    ser <- function(x_, y) {
        tvals <- abs(coef(summary(x_))[, 3])
        datares <- datasub
        lmres <- x_                               # correction: in case of all tvals > thresh
        while (min(tvals) < thresh) {
          #  datares <- datares[, -1 * which.min(tvals)]
            datares[,which.min(tvals)] <- NULL    # correction: in case of all tvals < thresh     
            lmres <- lm(y ~ -1 + ., data = datares)
            tvals <- abs(coef(summary(lmres))[, 3])
            if (ncol(datares) == 1) {
                break
            }
        }
        return(list(lmres = lmres, datares = datares))
    }
    if (method == "ser") {
        x$restrictions <- matrix(0, nrow = K, ncol = ncol(datasub))
        colnames(x$restrictions) <- namesall
        rownames(x$restrictions) <- colnames(yendog)
        for (i in 1:K) { # i=1
            temp <- ser(x$varresult[[i]], yendog[, i])   
            x$varresult[[i]] <- temp[[1]]
            namessub <- colnames(temp[[2]])
            x$restrictions[i, namesall %in% namessub] <- 1
        }
    }
    else if (method == "manual") {
        resmat <- as.matrix(resmat)
        if (!(nrow(resmat) == K) | !(ncol(resmat) == ncol(datasub))) {
            stop(paste("\n Please provide resmat with dimensions:", 
                K, "x", ncol(datasub), "\n"))
        }
        x$restrictions <- resmat
        colnames(x$restrictions) <- namesall
        rownames(x$restrictions) <- colnames(yendog)
        for (i in 1:K) {
            datares <- datasub[, which(x$restrictions[i, ] == 
                1)]
            y <- yendog[, i]
            lmres <- lm(y ~ -1 + ., data = datares)
            x$varresult[[i]] <- lmres
        }
    }
    return(x)
}



####################################################
#
#
#
#                     D I R E C T
#
#
#
####################################################

dirVARselect <- function (y, lag.max = 10, hor=12,type = c("const"), season = NULL, exogen = NULL)
{
    y <- as.matrix(y)
    if (any(is.na(y)))
        stop("\nNAs in y.\n")
    colnames(y) <- make.names(colnames(y))
    K <- ncol(y)
    lag.max <- abs(as.integer(lag.max))
#    type <- match.arg(type)
    lag <- abs(as.integer(lag.max + hor))        # modified
    ylagged <- embed(y, lag)[, -c(1:(K*hor))]    # modified
    yendog <- y[-c(1:(lag.max+hor-1)), ]         # modified
    # cbind(yendog,ylagged)
    sample <- nrow(ylagged)
    rhs <- switch(type,
        const = rep(1, sample),
        trend = seq(lag.max+1, length = sample),
        both = cbind(rep(1, sample), seq(lag.max +1, length = sample)),
        none = NULL)

    idx <- seq(K, K * lag.max, K)
    if (!is.null(rhs)) {
        detint <- ncol(as.matrix(rhs))
    } else {
        detint <- 0
    }
    criteria <- matrix(NA, nrow = 4, ncol = lag.max)
    rownames(criteria) <- c("AIC(n)", "HQ(n)", "SC(n)", "FPE(n)")
    colnames(criteria) <- paste(seq(1:lag.max))
    for (i in 1:lag.max) {   # i=2
        ys.lagged <- cbind(ylagged[, c(1:idx[i])], rhs)
        sampletot <- nrow(y)
        nstar <- ncol(ys.lagged)
        resids <- resid(lm(yendog ~ -1 + ys.lagged))
        sigma.det <- det(crossprod(resids)/sample)
        criteria[1, i] <- log(sigma.det) + (2/sample) * (i *
            K^2 + K * detint)
        criteria[2, i] <- log(sigma.det) + (2 * log(log(sample))/sample) *
            (i * K^2 + K * detint)
        criteria[3, i] <- log(sigma.det) + (log(sample)/sample) *
            (i * K^2 + K * detint)
        criteria[4, i] <- ((sample + nstar)/(sample - nstar))^K *
            sigma.det
    }
    order <- apply(criteria, 1, which.min)
    return(list(selection = order, criteria = criteria,hor=hor))
}

####################################################
# original function 'VAR' in package 'vars'
# modified for direct forecasting by Nikolay Robinzonov
# Date: 2007-08-06
#     'hor' is an extra parameter
####################################################

dirVAR <- function (y, p = 2, hor=3, type = c("const"),
    season = NULL, exogen = NULL)
{
    y <- as.matrix(y)
    if (any(is.na(y)))
        stop("\nNAs in y.\n")
    if (ncol(y) < 2)
        stop("The matrix 'y' should contain at least two variables. For univariate analysis consider ar() and arima() in package stats.\n")
    if (is.null(colnames(y))) {
        colnames(y) <- paste("y", 1:ncol(y), sep = "")
        warning(paste("No column names supplied in y, using:",
            paste(colnames(y), collapse = ", "), ", instead.\n"))
    }
    colnames(y) <- make.names(colnames(y))
    y.orig <- y
#    type <- match.arg(type)
    obs <- dim(y)[1]
    K <- dim(y)[2]
    sample <- obs - p - hor + 1                           # modified
    ylags <- embed(y, dimension = p+hor)[, -(1:(K*hor))]  # modified
    temp1 <- NULL
    for (i in 1:p) {
        temp <- paste(colnames(y), ".l", i, sep = "")
        temp1 <- c(temp1, temp)
    }
    colnames(ylags) <- temp1
    yend <- y[-c(1:(p+hor-1)), ]                          # modified
    # cbind(yend,ylags)
    if (type == "const") {
        rhs <- cbind(ylags, rep(1, sample))
        colnames(rhs) <- c(colnames(ylags), "const")
    }

    datamat <- as.data.frame(rhs)
    colnames(datamat) <- colnames(rhs)
    equation <- list()
    for (i in 1:K) {
        y <- yend[, i]
        equation[[colnames(yend)[i]]] <- lm(y ~ -1 + ., data = datamat)
    }
    result <- list(varresult = equation, datamat = data.frame(cbind(yend,rhs)),
                   y = y.orig, type = type, p = p, K = K, obs = sample,
        totobs = sample + p, restrictions = NULL, call = match.call())
    class(result) <- "varest"
    return(result)
}
####################################################
