##########################################################
### wrapper function to save graphics
###########################################################
dev.on <- function(myname, width = 14, height = 10,...){
  # check the extension 
  eps <- regexpr("\\.ps$",myname)!= -1 || regexpr("\\.eps$",myname)!= -1
  pdf <- regexpr("\\.pdf$",myname)!= -1
  if(!any(c(eps,pdf))) stop("Provide suitable extension.\n")

  if(eps)
    postscript(myname, width = width, height = height,
               onefile = FALSE, paper = "special", horizontal=F, 
               family = "Times", encoding = "ISOLatin1")

  if(pdf)
    pdf(myname, width = width, height = height,
        onefile = FALSE, paper = "special",
        family = "Times", encoding = "ISOLatin1")
 
}            
# family - "Times", "AvantGarde", "Bookman", "Courier", "NewCenturySchoolbook", "Palatino"


###########################################################
### define some log functions
###########################################################
zeit    <- function(){format(Sys.time(),"%Y-%m-%d %X")}
logging <- function(mytext,mypath) cat(mytext,file=paste(mypath,"/log.txt",sep=""),fill=T, append=T)
logheader <- function(d,n,parset){
  msg0 <- paste(names(parset),"=",unlist(parset),collapse=", ")
  msg1 <- paste("<!>",msg0," | max.lag =",d,", n =",n)
  msg1
 }
 

###########################################################
### my embed version
###########################################################
embed2 <- function(y,dimension=1,hor=1){
  y <- as.matrix(y)
  K <- ncol(y)
  if(NROW(y) < (dimension+hor)) stop("Not enough observations.")
  if(!is.null(colnames(y)))
    colnames(y) <- names <- make.names(colnames(y))
  else
    colnames(y) <- names <- paste("Series",1:K,sep="")
  lag <- abs(as.integer(dimension)) + hor
  if(hor==0)
   xlagged <- embed(y,lag)
  else
   xlagged <- embed(y,lag)[,-c(1:(K*hor))]
  ytruncated  <- embed(y,lag)[,c(1:K)]
  names.L <- paste(names,".L",rep(1:dimension,each=K),sep="")
  res <- cbind(ytruncated,xlagged)
  colnames(res) <- c(names,names.L)
  as.data.frame(res)
}

# example
# x = paste(rep(c("x","y","z"),each=20),1:20,sep="")
# xx = matrix(x,ncol=3); colnames(b) <- c("a","b","c"); b
# embed2(b)
# embed2(b,3,2)

###########################################################
### build formula for mboost with 'p-spline' base learner
###########################################################
bbsformula <- function(dat, df.arg = 4, knots.arg = 20, contr = NULL) {
  names <- names(dat)
  response <- names[1]
  covariates <- names[-1]
  if(!is.null(contr)) {
    if(!is.character(contr)) stop("'contr' must be of class character.")
    if(!contr %in% covariates) stop("There is no ",contr," among your covariates.")
    fcontr <- paste("bols(",contr,", contrasts.arg = 'contr.sum' )", sep = "")
    covariates <- covariates[!covariates %in% contr]
  }

  pred  <- paste("bbs(",covariates, ", df = ", df.arg ,", knots = ",knots.arg,")", sep="", collapse="+")
  if(!is.null(contr))
      pred <- paste(pred, fcontr, sep=" + ")
  model <- as.formula(paste(response,"~",pred)) 
  model
}


###########################################################
### Generate 'n.ahead' out-of-sample forecasts
###########################################################
myfcst <- function(fit,newx,n.ahead) {
 prog = c()
 names = names(newx)
 l = length(newx)
 cpnew <- as.matrix(newx)

if( length(class(fit))==1 && class(fit)=="stepwise_HaY") {
   fit = fit$fit
   cpnew <- as.data.frame(newx)
   if(length(fit$coefficients)==1 && "(Intercept)"%in%names(fit$coefficients))
   return(as.numeric(rep(fit$coefficients,n.ahead)))
}

if( any(class(fit)%in%"glmboost")) {
   cpnew <- as.data.frame(newx)
}


for(h in 1:n.ahead){ # h=1
   pr <- predict(fit,cpnew)
   cpnew[2:l] <- cpnew[1:(l-1)]
   cpnew[1] = pr
   if(abs(pr)>5) pr=0
   prog <- c(prog,pr)
  }

as.numeric(prog)

}




################################################################
## Usage:  Print row min/max positions in boldface for latex
## Author: Nikolay.Robinzonov@stat.uni-muenchen.de
################################################################
boldify <- function(tab, bld = NULL, cnames = TRUE, rnames = TRUE,
                    color = "black", NA.char = " ")
{ 
  if(!class(tab) %in% c("matrix","data.frame"))
    stop("You must provide either 'matrix' or 'data.frame'.\n")
  
  ## column names
  if(cnames){
    if(length(colnames(tab)) == 0)
      warning("No 'colnames' provided but still required.")
    frn <- ifelse(rnames, "&","")
    cat(frn, paste(colnames(tab), collapse = " & "), "\\\\ \\hline \n")
  }
  
  ## find the min/max positions 
  ibld <- 1:ncol(tab)                   # nothing will happen if bld == NULL
  char.cols <- rep(FALSE,ncol(tab))     # no character columns if bld == NULL
  if(!is.null(bld)){
    ## data.frames could be tricky because of character columns (find them)
    char.cols <- do.call("c",lapply(as.data.frame(tab),'class')) %in% "character"
    work <- tab[,!char.cols, drop = FALSE]
    for(i in 1:nrow(work))
      ibld[i] <- ifelse(bld == "min", which.min(work[i,]), which.max(work[i,]))
    for(i in which(char.cols))
      ibld[ibld >= i] <- ibld[ibld >= i] + 1
  }
  
  mypaste <- function(x) {
    res <- x
    for(i in 1:ncol(x)){
      res[,i] <- format(x[,i])
    }
    res
  }
  
  xtab <- mypaste(tab)  
  mcat <- function(i) {
    p <- ibld[i]
    lx <- as.character(xtab[i,])
    lx[is.na(tab[i,])] <- NA.char       # NA's
    x <- lx[p]
    xx <- paste("\\textcolor{",color,"}{\\textbf{",x,"}}", sep="")
    lx[p] <- ifelse(is.null(bld),x,xx)
    lx <- paste(lx, collapse=" & ")
    rn <- ifelse(rnames, paste(rownames(tab)[i], " &"),"")
    cat(rn, lx, " \\\\ \n")
  }
  
  tmp <- sapply(1:nrow(tab), mcat)
}

#################### examples ####################
## set.seed(2485); (tab <- round(matrix(rnorm(60, sd = 10), 6),2))
## colnames(tab) <- LETTERS[1:ncol(tab)]; rownames(tab) <- letters[1:nrow(tab)]
## boldify(tab)
## boldify(tab, "max", color = "red")
## boldify(tab, rnames = FALSE, cnames = FALSE, color="black")
##################################################
